// AudioTester_sinus_generator_FAST.ino
// ESP32
// IDE 1.8.12

#include "soc/rtc_io_reg.h"
#include "soc/rtc_cntl_reg.h"
#include "soc/sens_reg.h"
#include "soc/rtc.h"
#include "driver/dac.h"


int clk_8m_div = 0;         // 0 ... 7: RTC 8M clock divider (division is by clk_8m_div+1, i.e. 0 -> 8MHz frequency)
int frequency_step = 1;     // 1: 133 Hz ... 200: 26.500 Hz: Frequency for CW generator
int scale = 1;              // 0: 100 % - 1: 50% - 2: 25% - 3: 12.5% of full scale
int offset = 0;             // 0 = no offset
int invert = 2;             // invert MSB for correct sine waveform

int freq_array[] = {1, 2, 3, 5, 8, 10, 15, 18, 25, 30, 50, 80, 90, 92, 93, 95, 98, 100, 102, 105, 110, 115, 121, 123, 125};

void setup()
{ Serial.begin(250000);
}

void loop()
{ for (int n = 1;  n <= 23; n++)
  {  frequency_step = freq_array[n];
     
     Serial.println(frequency_step);
     
     // Enable generator
     SET_PERI_REG_MASK(SENS_SAR_DAC_CTRL1_REG, SENS_SW_TONE_EN);
     // connect generator to this channel
     SET_PERI_REG_MASK(SENS_SAR_DAC_CTRL2_REG, SENS_DAC_CW_EN1_M);
     // Invert MSB, otherwise part of waveform is inverted
     SET_PERI_REG_BITS(SENS_SAR_DAC_CTRL2_REG, SENS_DAC_INV1, 2, SENS_DAC_INV1_S);
    
     dac_output_enable(DAC_CHANNEL_1);
     // set SINE generator registers
     REG_SET_FIELD(RTC_CNTL_CLK_CONF_REG, RTC_CNTL_CK8M_DIV_SEL, clk_8m_div);
     SET_PERI_REG_BITS(SENS_SAR_DAC_CTRL1_REG, SENS_SW_FSTEP,    frequency_step, SENS_SW_FSTEP_S);
     SET_PERI_REG_BITS(SENS_SAR_DAC_CTRL2_REG, SENS_DAC_SCALE1,  scale,  SENS_DAC_SCALE1_S);
     SET_PERI_REG_BITS(SENS_SAR_DAC_CTRL2_REG, SENS_DAC_DC1,     offset, SENS_DAC_DC1_S);
     SET_PERI_REG_BITS(SENS_SAR_DAC_CTRL2_REG, SENS_DAC_INV1,    invert,   SENS_DAC_INV1_S);
  
     float frequency = RTC_FAST_CLK_FREQ_APPROX / (1 + clk_8m_div) * (float) frequency_step / 65536;
     vTaskDelay(2000/portTICK_PERIOD_MS); 
          
     delay(3000);
  }
}
